/**************************************************************************
**       Title: simple RGB image
**    $RCSfile: SimpleRGBImage.hh,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2002/11/02 12:07:11 $
**   Copyright: GPL $Author: pigorsch $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: SimpleRGBImage.hh,v $
**  Revision 1.1  2002/11/02 12:07:11  pigorsch
**  - initial revision
**
**  Revision 1.7  2002/10/30 15:46:06  pigorsch
**  - modified comments
**
**  Revision 1.6  2002/10/30 15:45:16  pigorsch
**  - changed image format from P6 to P3
**
**  Revision 1.5  2002/10/30 15:35:52  pigorsch
**  - added operator=
**
**  Revision 1.3  2002/10/30 15:19:00  pigorsch
**  - added Copy Constructor
**
**  Revision 1.2  2002/10/25 10:13:59  pigorsch
**  *** empty log message ***
**
**  Revision 1.1  2002/10/24 10:11:43  pigorsch
**  *** empty log message ***
**
**  Revision 1.2  2001/04/26 09:21:46  haasdonk
**  slight modifications
**
**
**************************************************************************/

#ifndef SIMPLERGBIMAGE_HH
#define SIMPLERGBIMAGE_HH

#include <iostream>
#include "RGBColor.hh"
#include "MyError.hh"


/*======================================================================*/
/*!
 *  \class SimpleRGBImage  SimpleRGBImage.hh
 *  \brief The SimpleRGBImage class is a very basic image class for
 *  storing and manipulating RGB images.
 *
 *  It contains methods for loading and saving PPM images and for
 *  direct access to the pixels
 */
/*======================================================================*/
class SimpleRGBImage
{

  static const unsigned int MAX_DIMENSION = 16384;

public:
  /*-----------------------------------------------------------------------
   *  Error classes 
   *-----------------------------------------------------------------------*/
  class ReadError: public MyError 
  {
  public: 
    ReadError(std::string s): 
            MyError(s)
          {};
  };

  class WriteError: public MyError 
  {
  public: 
    WriteError(std::string s):
            MyError(s)
          {};
  };

  /*======================================================================*/
  /*! 
   *   Default Constructor. Creates an empty image
   */
  /*======================================================================*/
  SimpleRGBImage();

  /*======================================================================*/
  /*! 
   *   Copy Constructor.
   */
  /*======================================================================*/
  SimpleRGBImage(const SimpleRGBImage& image);
  
  /*======================================================================*/
  /*! 
   *   Size Constructor. Creates an image of arbitrary dimensions
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  SimpleRGBImage(unsigned int width, unsigned int height);

  /*======================================================================*/
  /*! 
   *   Load Constructor. Loads an image from an input stream. 
   *   See read-function for details.
   *
   *   \param is   istream to load image from
   */
  /*======================================================================*/
  SimpleRGBImage(std::istream& is);

  /*======================================================================*/
  /*! 
   *   Destructor
   */
  /*======================================================================*/
  ~SimpleRGBImage();
  
  /*======================================================================*/
  /*! 
   *   resize image to width*height. If it has already this size, nothing
   *   is done 
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  void resize(unsigned int width, unsigned int height);
  

  /*======================================================================*/
  /*! 
   *   read an image from the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param is   input stream. Must be already opened for reading,
   *               with the read postition pointing to the begin of
   *               the image header 
   *
   *   \exception ReadError an error occured during
   *                        reading. ReadError::msg() contains the plain
   *                        text error message.
   */
  /*======================================================================*/
  void read(std::istream& is);
  
  /*======================================================================*/
  /*! 
   *   write the image to the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param os  output stream. Must be already opened for writing
   *
   *   \exception WriteError an error occured during
   *                         writing. WriteError::msg() contains the plain
   *                         text error message.
   */
  /*======================================================================*/
  void write(std::ostream& os);
  

  /*======================================================================*/
  /*! 
   *   unchecked access to a pixel (two-dimensional position)
   *
   *   \param x    horizontal pixel coordinate
   *   \param y    vertical pixel coordinate
   *
   *   \return  reference to the pixel at position row,col
   */
  /*======================================================================*/
  RGBColor&       operator()(unsigned int x, unsigned int y);
  const RGBColor& operator()(unsigned int x, unsigned int y) const;
  


  /*======================================================================*/
  /*! 
   *   width of the image
   *
   *   \return image width
   */
  /*======================================================================*/
  unsigned int width() const;

  
  /*======================================================================*/
  /*! 
   *   height of the image
   *
   *   \return image height
   */
  /*======================================================================*/
  unsigned int height() const;

  /*======================================================================*/
  /*! 
   *   size of the image (number of pixels)
   *
   *   \return number of pixels
   */
  /*======================================================================*/
  size_t size() const;

  /*======================================================================*/
  /*! 
   *   begin of image
   *
   *   \return pointer to first RGBColor of image
   */
  /*======================================================================*/
  RGBColor*       begin();
  const RGBColor* begin() const;

  /*======================================================================*/
  /*! 
   *   end of image
   *
   *   \return pointer to the first RGBColor BEHIND the last RGBColor 
   *           of the image
   */
  /*======================================================================*/
  RGBColor*       end();
  const RGBColor* end() const;

  /*======================================================================*/
  /*! 
   *   operator[] 
   *
   *   \param n    pixel number
   *   \return reference to RGBColor number n of image
   */
  /*======================================================================*/
  RGBColor&       operator[](size_t n); 
  const RGBColor& operator[](size_t n) const;


  /*======================================================================*/
  /*! 
   *   operator=
   *
   *   \param image    image to copy
   *   \return reference to modified image
   */
  /*======================================================================*/
  SimpleRGBImage& operator=(const SimpleRGBImage& image);
  
private:
  unsigned int _width, _height, _size;
  RGBColor* _pixels;
  RGBColor* _pixelsEnd;
  RGBColor** _rowStart;
};

#include "SimpleRGBImage.icc"

#endif
