/**************************************************************************
**       Title: test program for RGBSegment
**    $RCSfile: testRGBSegment.cc,v $
**   $Revision: 1.4 $$Name:  $
**       $Date: 2005/05/02 11:25:37 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: testRGBSegment.cc,v $
**
**
**************************************************************************/

#include <iostream>
#include <fstream>

#include "MyError.hh"
#include "IntImage.hh"
#include "SimpleRGBImage.hh"
#include "RGBColor.hh"
#include "RGBClassifier.hh"
#include "ConnectedComponentLabeling.hh"

/*======================================================================*/
/*! 
 *   Check if the default constructor works.
 */
/*======================================================================*/
static bool testEmptyContructor()
{
  std::cout << "testEmptyContructor\n";
  
  IntImage image;
  
  if (image.width()!=0 || image.height()!=0 || 
      image.size()!=0 ||
      image.begin()!=image.end())
  {
    std::cout << "[properties] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*! 
 *   Check if the copy constructor works.
 */
/*======================================================================*/
static bool testCopyConstructor()
{
  std::cerr << "testCopyConstructor\n";
  
  // create 0 image
  IntImage image1(5, 5);
  for (int* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=0;
  }
  
  // copy image
  IntImage image2(image1);

  // check if both images have equal properties
  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }
  
  // check if pointers are different
  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  // check if copying also copied pixels
  for (const int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=0)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }
  
  // change one pixel in new image
  image2(3, 3)=1;
  
  // check if pixel changed not in original image but in new one
  if (image1(3, 3)!=0 || image2(3, 3)!=1)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}  

/*======================================================================*/
/*! 
 *   Check if the size constructor works.
 */
/*======================================================================*/
static bool testSizeConstructor()
{
  std::cout << "testSizeConstructor\n";
  
  // create image of arbitrary size
  IntImage image(42, 17);
  // check if properties are correct
  if (image.width()!=42 || image.height()!=17 || image.size()!=42*17 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }
  
  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*! 
 *   Check if resize() works.
 */
/*======================================================================*/
static bool testResize()
{
  std::cout << "testResize\n";
  
  // create image
  IntImage image(42, 17);
  // resize image
  image.resize(117, 5);
  // check if properties are correct
  if (image.width()!=117 || image.height()!=5 || image.size()!=117*5 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }
  
  // set color of arbitrary pixel
  image(116, 4)=0;
  // try to access that pixel by op[]
  if (image[117*5-1]!=0)
  {
    std::cerr << "[pixel access] failed\n";
    return true;
  }
  
  // try to make image really huge
  try
  {
    image.resize(1000000, 1000000);
  }
  catch (const MyError& e) // correct exception thrown
  {
  }
  catch (...) // probably bad_alloc exception
  {
    std::cerr << "[oversize] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}
 
/*======================================================================*/
/*! 
 *   Check if operator=() works, see testCopyConstructor()
 */
/*======================================================================*/
static bool testAssignmentOperator()
{
  std::cerr << "testAssignmentOperator\n";
 
  IntImage image1(5, 5);
  for (int* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=0;
  }
  
  IntImage image2(3, 3);
  image2=image1;

  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }
  
  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  for (const int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=0)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }

  for (int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    *p=1;
  }


  if (image1(3, 3)!=0 || image2(3, 3)!=1)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }
  
  std::cerr << "ok\n";
  return true;
}


/*--------------------------------------------------
*  test RGB Classifier
----------------------------------------------------*/
static bool testRGBClassifier()
{
    std::cerr<<"testing RGBClassifier ...\n";
    SimpleRGBImage in(5,5);
    IntImage* out; 
    RGBClassifier classifier(RGBColor::Red, 4, 3, 5);
    RGBColor test = RGBColor::Red;
    test.set(test.red()-3,test.green()+1,test.blue());
    in[4] = test;
    test.set(test.red()-3,test.green()+4,test.blue());
    in[5] = test;
    
    out = classifier.classify(in);
    if (out->operator[](4) != 1)
    {
	std::cerr << out->operator[](4) <<" FAILURE!\n";
	return false;
	    
    }
    if (out->operator[](5) != 0)
    {
	std::cerr << out->operator[](5) <<" FAILURE!\n";
	return false;
    }
    std::cerr << "ok\n";
    return true;	


}

/*---------------------------------------------------
 * test ConnectedComponentLabeling
 * -------------------------------------------------*/
static bool testCCL()
{
    std::cerr<<"testing CCL ...\n";
    IntImage* image = new IntImage(10,10);
    for (int i=0;i<100;i++)
    {
    	image->operator[](i)=0;
    }
    image->operator()(2,2)=1;
    image->operator()(2,3)=1;
    image->operator()(5,7)=1;
    image->operator()(6,7)=1;
    image->operator()(8,8)=1;
    image->operator()(8,7)=1;

    ConnectedComponentLabeling MyLabel;
    MyLabel.label(*image);

    if ((image->operator()(2,3)==2)&&(image->operator()(6,7)==3)&&(image->operator()(8,7)==4))
    {
    	std::cerr << "ok\n";
    	return true;
    }
    else
    {
	std::cerr<<image->operator()(2,3)<<" "<<image->operator()(6,7)<<" "<<image->operator()(8,7)<<"\n";    
	std::cerr << "FAILED\n";
	return false;
    }
}



/*--------------------------------------------------
*  main
----------------------------------------------------*/
int main()
{
  try
  {    
    testEmptyContructor();
    testCopyConstructor();
    testSizeConstructor();
    testResize();  
    testAssignmentOperator();
    testRGBClassifier();
    testCCL();
  }
  catch (const MyError& e)
  {
    std::cerr << "MyError caught: " << e.msg() << std::endl;
  }
  catch (...)
  {
    std::cerr << "unknown exception caught\n";
  }
}
